@@ -124,7 +124,6 @@ struct ImagingMemoryInstance {
 
 struct ImagingAccessInstance {
     const char *mode;
-    void *(*line)(Imaging im, int x, int y);
     void (*get_pixel)(Imaging im, int x, int y, void *pixel);
     void (*put_pixel)(Imaging im, int x, int y, const void *pixel);
 };
@@ -143,6 +142,7 @@ struct ImagingPaletteInstance {
     char mode[IMAGING_MODE_LENGTH]; /* Band names */
 
     /* Data */
+    int size;
     UINT8 palette[1024]; /* Palette data (same format as image data) */
 
     INT16 *cache;   /* Palette cache (used for predefined palettes) */
@@ -290,7 +290,7 @@ ImagingConvertTransparent(Imaging im, const char *mode, int r, int g, int b);
 extern Imaging
 ImagingCrop(Imaging im, int x0, int y0, int x1, int y1);
 extern Imaging
-ImagingExpand(Imaging im, int x, int y, int mode);
+ImagingExpand(Imaging im, int x, int y);
 extern Imaging
 ImagingFill(Imaging im, const void *ink);
 extern int
@@ -317,7 +317,7 @@ ImagingMerge(const char *mode, Imaging bands[4]);
 extern int
 ImagingSplit(Imaging im, Imaging bands[4]);
 extern int
-ImagingGetBBox(Imaging im, int bbox[4]);
+ImagingGetBBox(Imaging im, int bbox[4], int alpha_only);
 typedef struct {
     int x, y;
     INT32 count;
@@ -370,7 +370,7 @@ ImagingTransform(
     int y0,
     int x1,
     int y1,
-    double *a,
+    double a[8],
     int filter,
     int fill);
 extern Imaging
@@ -487,7 +487,7 @@ ImagingDrawPieslice(
 extern int
 ImagingDrawPoint(Imaging im, int x, int y, const void *ink, int op);
 extern int
-ImagingDrawPolygon(Imaging im, int points, int *xy, const void *ink, int fill, int op);
+ImagingDrawPolygon(Imaging im, int points, int *xy, const void *ink, int fill, int width, int op);
 extern int
 ImagingDrawRectangle(
     Imaging im,
