@@ -32,6 +32,9 @@ def getrgb(color):
     :param color: A color string
     :return: ``(red, green, blue[, alpha])``
     """
+    if len(color) > 100:
+        msg = "color specifier is too long"
+        raise ValueError(msg)
     color = color.lower()
 
     rgb = colormap.get(color, None)
@@ -43,7 +46,7 @@ def getrgb(color):
 
     # check for known string formats
     if re.match("#[a-f0-9]{3}$", color):
-        return (int(color[1] * 2, 16), int(color[2] * 2, 16), int(color[3] * 2, 16))
+        return int(color[1] * 2, 16), int(color[2] * 2, 16), int(color[3] * 2, 16)
 
     if re.match("#[a-f0-9]{4}$", color):
         return (
@@ -54,7 +57,7 @@ def getrgb(color):
         )
 
     if re.match("#[a-f0-9]{6}$", color):
-        return (int(color[1:3], 16), int(color[3:5], 16), int(color[5:7], 16))
+        return int(color[1:3], 16), int(color[3:5], 16), int(color[5:7], 16)
 
     if re.match("#[a-f0-9]{8}$", color):
         return (
@@ -66,7 +69,7 @@ def getrgb(color):
 
     m = re.match(r"rgb\(\s*(\d+)\s*,\s*(\d+)\s*,\s*(\d+)\s*\)$", color)
     if m:
-        return (int(m.group(1)), int(m.group(2)), int(m.group(3)))
+        return int(m.group(1)), int(m.group(2)), int(m.group(3))
 
     m = re.match(r"rgb\(\s*(\d+)%\s*,\s*(\d+)%\s*,\s*(\d+)%\s*\)$", color)
     if m:
@@ -112,33 +115,43 @@ def getrgb(color):
 
     m = re.match(r"rgba\(\s*(\d+)\s*,\s*(\d+)\s*,\s*(\d+)\s*,\s*(\d+)\s*\)$", color)
     if m:
-        return (int(m.group(1)), int(m.group(2)), int(m.group(3)), int(m.group(4)))
-    raise ValueError(f"unknown color specifier: {repr(color)}")
+        return int(m.group(1)), int(m.group(2)), int(m.group(3)), int(m.group(4))
+    msg = f"unknown color specifier: {repr(color)}"
+    raise ValueError(msg)
 
 
 def getcolor(color, mode):
     """
-    Same as :py:func:`~PIL.ImageColor.getrgb`, but converts the RGB value to a
-    greyscale value if the mode is not color or a palette image. If the string
-    cannot be parsed, this function raises a :py:exc:`ValueError` exception.
+    Same as :py:func:`~PIL.ImageColor.getrgb` for most modes. However, if
+    ``mode`` is HSV, converts the RGB value to a HSV value, or if ``mode`` is
+    not color or a palette image, converts the RGB value to a greyscale value.
+    If the string cannot be parsed, this function raises a :py:exc:`ValueError`
+    exception.
 
     .. versionadded:: 1.1.4
 
     :param color: A color string
-    :return: ``(graylevel [, alpha]) or (red, green, blue[, alpha])``
+    :param mode: Convert result to this mode
+    :return: ``(graylevel[, alpha]) or (red, green, blue[, alpha])``
     """
     # same as getrgb, but converts the result to the given mode
     color, alpha = getrgb(color), 255
     if len(color) == 4:
-        color, alpha = color[0:3], color[3]
+        color, alpha = color[:3], color[3]
 
-    if Image.getmodebase(mode) == "L":
+    if mode == "HSV":
+        from colorsys import rgb_to_hsv
+
+        r, g, b = color
+        h, s, v = rgb_to_hsv(r / 255, g / 255, b / 255)
+        return int(h * 255), int(s * 255), int(v * 255)
+    elif Image.getmodebase(mode) == "L":
         r, g, b = color
         # ITU-R Recommendation 601-2 for nonlinear RGB
         # scaled to 24 bits to match the convert's implementation.
         color = (r * 19595 + g * 38470 + b * 7471 + 0x8000) >> 16
         if mode[-1] == "A":
-            return (color, alpha)
+            return color, alpha
     else:
         if mode[-1] == "A":
             return color + (alpha,)
